@@ -19,6 +19,7 @@
 #include "mutt.h"
 #include "mutt_regex.h"
 #include "mutt_curses.h"
+#include "mutt_idna.h"
 
 #include <string.h>
 #include <ctype.h>
@@ -210,6 +211,7 @@ void mutt_create_alias (ENVELOPE *cur, ADDRESS *iadr)
 {
   ALIAS *new, *t;
   char buf[LONG_STRING], prompt[SHORT_STRING], *pc;
+  char *err = NULL;
   char fixed[LONG_STRING];
   FILE *rc;
   ADDRESS *adr = NULL;
@@ -249,9 +251,9 @@ retry_name:
   
   if (mutt_check_alias_name (buf, fixed))
   {
-    switch (mutt_yesorno (_("Warning: This alias name may not work.  Fix it?"), 1))
+    switch (mutt_yesorno (_("Warning: This alias name may not work.  Fix it?"), M_YES))
     {
-      case 1:  
+      case M_YES:
       	strfcpy (buf, fixed, sizeof (buf));
 	goto retry_name;
       case -1: 
@@ -263,11 +265,15 @@ retry_name:
   new->self = new;
   new->name = safe_strdup (buf);
 
+  mutt_addrlist_to_local (adr);
+  
   if (adr)
     strfcpy (buf, adr->mailbox, sizeof (buf));
   else
     buf[0] = 0;
 
+  mutt_addrlist_to_idna (adr, NULL);
+  
   do
   {
     if (mutt_get_field (_("Address: "), buf, sizeof (buf), 0) != 0 || !buf[0])
@@ -278,6 +284,12 @@ retry_name:
     
     if((new->addr = rfc822_parse_adrlist (new->addr, buf)) == NULL)
       BEEP ();
+    if (mutt_addrlist_to_idna (new->addr, &err))
+    {
+      mutt_error (_("Error: '%s' is a bad IDN."), err);
+      mutt_sleep (2);
+      continue;
+    }
   }
   while(new->addr == NULL);
   
@@ -294,9 +306,9 @@ retry_name:
   new->addr->personal = safe_strdup (buf);
 
   buf[0] = 0;
-  rfc822_write_address (buf, sizeof (buf), new->addr);
+  rfc822_write_address (buf, sizeof (buf), new->addr, 1);
   snprintf (prompt, sizeof (prompt), _("[%s = %s] Accept?"), new->name, buf);
-  if (mutt_yesorno (prompt, 1) != 1)
+  if (mutt_yesorno (prompt, M_YES) != M_YES)
   {
     mutt_free_alias (&new);
     return;
@@ -323,7 +335,7 @@ retry_name:
       strfcpy (buf, new->name, sizeof (buf));
     fprintf (rc, "alias %s ", buf);
     buf[0] = 0;
-    rfc822_write_address (buf, sizeof (buf), new->addr);
+    rfc822_write_address (buf, sizeof (buf), new->addr, 0);
     write_safe_address (rc, buf);
     fputc ('\n', rc);
     fclose (rc);
