@@ -26,6 +26,8 @@
 #include "copy.h"
 #include "mx.h"
 #include "pager.h"
+#include "mutt_crypt.h"
+#include "mutt_idna.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -38,19 +40,6 @@
 #include "buffy.h"
 #endif
 
-
-
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
-
-
-#ifdef HAVE_SMIME
-#include "smime.h"
-#endif
-
-
-
 #include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -81,16 +70,13 @@ int mutt_display_message (HEADER *cur)
   mutt_parse_mime_message (Context, cur);
   mutt_message_hook (Context, cur, M_MESSAGEHOOK);
 
-#if defined (HAVE_PGP) || defined (HAVE_SMIME)
-  /* see if PGP is needed for this message.  if so, we should exit curses */
-  if (cur->security)
+  /* see if crytpo is needed for this message.  if so, we should exit curses */
+  if (WithCrypto && cur->security)
   {
     if (cur->security & ENCRYPT)
     {
-#ifdef HAVE_SMIME
       if (cur->security & APPLICATION_SMIME)
-	smime_getkeys (cur->env);
-#endif
+	crypt_smime_getkeys (cur->env);
       if(!crypt_valid_passphrase(cur->security))
 	return 0;
 
@@ -108,22 +94,17 @@ int mutt_display_message (HEADER *cur)
   
   if (cmflags & M_CM_VERIFY || cur->security & ENCRYPT)
   {
-#ifdef HAVE_PGP
     if (cur->security & APPLICATION_PGP)
     {
       if (cur->env->from)
-        pgp_invoke_getkeys (cur->env->from);
-
-      mutt_message _("Invoking PGP...");
+        crypt_pgp_invoke_getkeys (cur->env->from);
+      
+      crypt_invoke_message (APPLICATION_PGP);
     }
-#endif
 
-#ifdef HAVE_SMIME
     if (cur->security & APPLICATION_SMIME)
-      mutt_message _("Invoking OpenSSL...");
-#endif
+      crypt_invoke_message (APPLICATION_SMIME);
   }
-#endif /*  defined (HAVE_PGP) || defined (HAVE_SMIME) */
 
 
   mutt_mktemp (tempfile);
@@ -178,25 +159,26 @@ int mutt_display_message (HEADER *cur)
   safe_fclose (&fpfilterout);	/* XXX - check result? */
 
   
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-  /* update crypto information for this message */
-  cur->security |= crypt_query (cur->content);
+  if (WithCrypto)
+  {
+    /* update crypto information for this message */
+    cur->security |= crypt_query (cur->content);
   
-  /* Remove color cache for this message, in case there
-     are color patterns for both ~g and ~V */
-  cur->pair = 0;
-#endif
+    /* Remove color cache for this message, in case there
+       are color patterns for both ~g and ~V */
+    cur->pair = 0;
+  }
 
   if (builtin)
   {
     pager_t info;
-#ifdef HAVE_SMIME
 
-    if (cur->security & APPLICATION_SMIME && (cmflags & M_CM_VERIFY))
+    if (WithCrypto 
+        && (cur->security & APPLICATION_SMIME) && (cmflags & M_CM_VERIFY))
     {
       if (cur->security & GOODSIGN)
       {
-	if (!smime_verify_sender(cur))
+	if (!crypt_smime_verify_sender(cur))
 	  mutt_message ( _("S/MIME signature successfully verified."));
 	else
 	  mutt_error ( _("S/MIME certificate owner does not match sender."));
@@ -204,14 +186,12 @@ int mutt_display_message (HEADER *cur)
       else if (cur->security & SIGN || cur->security & BADSIGN)
 	mutt_error ( _("S/MIME signature could NOT be verified."));
     }
-#endif
 
-#ifdef HAVE_PGP
-    if ((cur->security & APPLICATION_PGP) && (cmflags & M_CM_VERIFY))
+    if (WithCrypto 
+        && (cur->security & APPLICATION_PGP) && (cmflags & M_CM_VERIFY))
       mutt_message ((cur->security & GOODSIGN) ?
 		    _("PGP signature successfully verified.") :
 		    _("PGP signature could NOT be verified."));
-#endif
 
     /* Invoke the builtin pager */
     memset (&info, 0, sizeof (pager_t));
@@ -248,6 +228,7 @@ void ci_bounce_message (HEADER *h, int *redraw)
   char prompt[SHORT_STRING];
   char buf[HUGE_STRING] = { 0 };
   ADDRESS *adr = NULL;
+  char *err = NULL;
   int rc;
 
   if(h)
@@ -274,27 +255,41 @@ void ci_bounce_message (HEADER *h, int *redraw)
 
   adr = mutt_expand_aliases (adr);
 
-  buf[0] = 0;
-  rfc822_write_address (buf, sizeof (buf), adr);
-
-#define extra_space (15 + 7 + 2)
-  /*
-   * This is the printing width of "...? ([y=yes]/n=no): ?" plus 2
-   * for good measure. This is not ideal. FIXME.
-   */
-  snprintf (prompt, sizeof (prompt) - 4,
-           (h ? _("Bounce message to %s") : _("Bounce messages to %s")), buf);
-  mutt_format_string (prompt, sizeof (prompt) - 4,
-		      0, COLS-extra_space, 0, 0,
-		      prompt, sizeof (prompt), 0);
-  strcat (prompt, "...?");	/* __STRCAT_CHECKED__ */
-  if (mutt_yesorno (prompt, 1) != 1)
+  if (mutt_addrlist_to_idna (adr, &err) < 0)
   {
+    mutt_error (_("Bad IDN: '%s'"), err);
+    FREE (&err);
     rfc822_free_address (&adr);
-    CLEARLINE (LINES-1);
     return;
   }
 
+  buf[0] = 0;
+  rfc822_write_address (buf, sizeof (buf), adr, 1);
+
+#define extra_space (15 + 7 + 2)
+  snprintf (prompt, sizeof (prompt),
+           (h ? _("Bounce message to %s") : _("Bounce messages to %s")), buf);
+
+  if (mutt_strwidth (prompt) > COLS - extra_space)
+  {
+    mutt_format_string (prompt, sizeof (prompt),
+			0, COLS-extra_space, 0, 0,
+			prompt, sizeof (prompt), 0);
+    strncat (prompt, "...?", sizeof (prompt));
+  }
+  else
+    strncat (prompt, "?", sizeof (prompt));
+
+  if (query_quadoption (OPT_BOUNCE, prompt) == M_NO)
+  {
+    rfc822_free_address (&adr);
+    CLEARLINE (LINES - 1);
+    mutt_message (h ? _("Message not bounced.") : _("Messages not bounced."));
+    return;
+  }
+
+  CLEARLINE (LINES - 1);
+  
   rc = mutt_bounce_message (NULL, h, adr);
   rfc822_free_address (&adr);
   /* If no error, or background, display message. */
@@ -328,15 +323,12 @@ void pipe_msg (HEADER *h, FILE *fp, int decode, int print)
   
   pipe_set_flags (decode, print, &cmflags, &chflags);
 
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-    if (decode && h->security & ENCRYPT)
-    {
-      if(!crypt_valid_passphrase(h->security))
-	return;
-      endwin ();
-    }
-#endif
-
+  if (WithCrypto && decode && h->security & ENCRYPT)
+  {
+    if(!crypt_valid_passphrase(h->security))
+      return;
+    endwin ();
+  }
 
   if (decode)
     mutt_parse_mime_message (Context, h);
@@ -369,15 +361,13 @@ static int _mutt_pipe_message (HEADER *h, char *cmd,
 
     mutt_message_hook (Context, h, M_MESSAGEHOOK);
 
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-    if (decode)
+    if (WithCrypto && decode)
     {
       mutt_parse_mime_message (Context, h);
       if(h->security & ENCRYPT && !crypt_valid_passphrase(h->security))
 	return 1;
     }
     mutt_endwin (NULL);
-#endif
 
     if ((thepid = mutt_create_filter (cmd, &fpout, NULL, NULL)) < 0)
     {
@@ -392,11 +382,7 @@ static int _mutt_pipe_message (HEADER *h, char *cmd,
   else
   { /* handle tagged messages */
 
-
-
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-
-    if (decode)
+    if (WithCrypto && decode)
     {
       for (i = 0; i < Context->vcount; i++)
 	if(Context->hdrs[Context->v2r[i]]->tagged)
@@ -408,7 +394,6 @@ static int _mutt_pipe_message (HEADER *h, char *cmd,
 	    return 1;
 	}
     }
-#endif
     
     if (split)
     {
@@ -615,9 +600,16 @@ void mutt_display_address (ENVELOPE *env)
   adr = mutt_get_address (env, &pfx);
 
   if (!adr) return;
-
+  
+  /* 
+   * Note: We don't convert IDNA to local representation this time.
+   * That is intentional, so the user has an opportunity to copy &
+   * paste the on-the-wire form of the address to other, IDN-unable
+   * software. 
+   */
+  
   buf[0] = 0;
-  rfc822_write_address (buf, sizeof (buf), adr);
+  rfc822_write_address (buf, sizeof (buf), adr, 0);
   mutt_message ("%s: %s", pfx, buf);
 }
 
@@ -626,30 +618,24 @@ static void set_copy_flags (HEADER *hdr, int decode, int decrypt, int *cmflags,
   *cmflags = 0;
   *chflags = CH_UPDATE_LEN;
   
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-  if (!decode && decrypt && (hdr->security & ENCRYPT))
+  if (WithCrypto && !decode && decrypt && (hdr->security & ENCRYPT))
   {
-#ifdef HAVE_PGP
-    if (mutt_is_multipart_encrypted(hdr->content))
+    if ((WithCrypto & APPLICATION_PGP)
+        && mutt_is_multipart_encrypted(hdr->content))
     {
       *chflags = CH_NONEWLINE | CH_XMIT | CH_MIME;
       *cmflags = M_CM_DECODE_PGP;
     }
-    else if (mutt_is_application_pgp (hdr->content) & ENCRYPT)
+    else if ((WithCrypto & APPLICATION_PGP)
+              && mutt_is_application_pgp (hdr->content) & ENCRYPT)
       decode = 1;
-#endif
-#if defined(HAVE_PGP) && defined(HAVE_SMIME)
-    else
-#endif
-#ifdef HAVE_SMIME
-	  if (mutt_is_application_smime(hdr->content) & ENCRYPT)
+    else if ((WithCrypto & APPLICATION_SMIME)
+             && mutt_is_application_smime(hdr->content) & ENCRYPT)
     {
       *chflags = CH_NONEWLINE | CH_XMIT | CH_MIME;
       *cmflags = M_CM_DECODE_SMIME;
     }
-#endif
   }
-#endif /* defined(HAVE_PGP) || defined(HAVE_SMIME) */
 
   if (decode)
   {
@@ -690,9 +676,7 @@ int mutt_save_message (HEADER *h, int delete,
 		       int decode, int decrypt, int *redraw)
 {
   int i, need_buffy_cleanup;
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
   int need_passphrase = 0, app=0;
-#endif
   char prompt[SHORT_STRING], buf[_POSIX_PATH_MAX];
   CONTEXT ctx;
   struct stat st;
@@ -716,10 +700,11 @@ int mutt_save_message (HEADER *h, int delete,
 
   if (h)
   {
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+    if (WithCrypto)
+    {
       need_passphrase = h->security & ENCRYPT;
       app = h->security;
-#endif
+    }
     mutt_message_hook (Context, h, M_MESSAGEHOOK);
     mutt_default_save (buf, sizeof (buf), h);
   }
@@ -741,10 +726,11 @@ int mutt_save_message (HEADER *h, int delete,
     {
       mutt_message_hook (Context, h, M_MESSAGEHOOK);
       mutt_default_save (buf, sizeof (buf), h);
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-      need_passphrase = h->security & ENCRYPT;
-      app = h->security;
-#endif
+      if (WithCrypto)
+      {
+        need_passphrase = h->security & ENCRYPT;
+        app = h->security;
+      }
       h = NULL;
     }
   }
@@ -778,10 +764,9 @@ int mutt_save_message (HEADER *h, int delete,
   if (!mutt_save_confirm (buf, &st))
     return -1;
 
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-  if (need_passphrase && (decode || decrypt) && !crypt_valid_passphrase(app))
+  if (WithCrypto && need_passphrase && (decode || decrypt)
+      && !crypt_valid_passphrase(app))
     return -1;
-#endif
   
   mutt_message (_("Copying to %s..."), buf);
   
@@ -904,9 +889,11 @@ void mutt_edit_content_type (HEADER *h, BODY *b, FILE *fp)
 
   if (!h && b->type == TYPETEXT && charset_changed)
   {
+    int r;
     snprintf (tmp, sizeof (tmp), _("Convert to %s upon sending?"),
 	      mutt_get_parameter ("charset", b->parameter));
-    b->noconv = !mutt_yesorno (tmp, !b->noconv);
+    if ((r = mutt_yesorno (tmp, !b->noconv)) != -1)
+      b->noconv = (r == M_NO);
   }
 
   /* inform the user */
@@ -931,19 +918,16 @@ void mutt_edit_content_type (HEADER *h, BODY *b, FILE *fp)
   if (fp && (is_multipart (b) || mutt_is_message_type (b->type, b->subtype)))
     mutt_parse_part (fp, b);
   
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-  if (h)
+  if (WithCrypto && h)
   {
     if (h->content == b)
       h->security  = 0;
 
     h->security |= crypt_query (b);
   }
-#endif
 }
 
 
-#ifdef HAVE_PGP
 static int _mutt_check_traditional_pgp (HEADER *h, int *redraw)
 {
   MESSAGE *msg;
@@ -952,7 +936,7 @@ static int _mutt_check_traditional_pgp (HEADER *h, int *redraw)
   mutt_parse_mime_message (Context, h);
   if ((msg = mx_open_message (Context, h->msgno)) == NULL)
     return 0;
-  if (pgp_check_traditional (msg->fp, h->content, 0))
+  if (crypt_pgp_check_traditional (msg->fp, h->content, 0))
   {
     h->security = crypt_query (h->content);
     *redraw |= REDRAW_FULL;
@@ -979,4 +963,4 @@ int mutt_check_traditional_pgp (HEADER *h, int *redraw)
   return rv;
 }
 
-#endif
+
