@@ -22,20 +22,15 @@
 #include "copy.h"
 #include "rfc2047.h"
 #include "mime.h"
-
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
-
-#ifdef HAVE_SMIME
-#include "smime.h"
-#endif
+#include "mutt_crypt.h"
+#include "mutt_idna.h"
 
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <unistd.h> /* needed for SEEK_SET under SunOS 4.1.4 */
 
+static int address_header_decode (char **str);
 static int copy_delete_attach (BODY *b, FILE *fpin, FILE *fpout, char *date);
 
 /* Ok, the only reason for not merging this with mutt_copy_header()
@@ -54,6 +49,7 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
   char **headers;
   int hdr_count;
   int x;
+  char *this_one = NULL;
   int error;
 
   if (ftell (in) != off_start)
@@ -141,6 +137,28 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
     /* Is it the begining of a header? */
     if (nl && buf[0] != ' ' && buf[0] != '\t')
     {
+      /* Do we have anything pending? */
+      if (this_one)
+      {
+	if (flags & CH_DECODE) 
+	{
+	  if (!address_header_decode (&this_one))
+	    rfc2047_decode (&this_one);
+	}
+	
+	if (!headers[x])
+	  headers[x] = this_one;
+	else 
+	{
+	  safe_realloc ((void **) &headers[x], mutt_strlen (headers[x]) + 
+			mutt_strlen (this_one) + sizeof (char));
+	  strcat (headers[x], this_one); /* __STRCAT_CHECKED__ */
+	  FREE (&this_one);
+	}
+	
+	this_one = NULL;
+      }
+      
       ignore = 1;
       this_is_from = 0;
       if (!from && mutt_strncmp ("From ", buf, 5) == 0)
@@ -188,32 +206,55 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
 	  }
 	}
       }
-
+      
       ignore = 0;
     } /* If beginning of header */
 
     if (!ignore)
     {
       dprint (2, (debugfile, "Reorder: x = %d; hdr_count = %d\n", x, hdr_count));
-      /* Save the header in headers[x] */
-      if (!headers[x])
-	headers[x] = safe_strdup (buf);
+      if (!this_one)
+	this_one = safe_strdup (buf);
       else
       {
-	safe_realloc ((void **) &headers[x],
-		      mutt_strlen (headers[x]) + mutt_strlen (buf) + sizeof (char));
-	strcat (headers[x], buf);	/* __STRCAT_CHECKED__ */
+	safe_realloc ((void **) &this_one, 
+		      mutt_strlen (this_one) + mutt_strlen (buf) + sizeof (char));
+	strcat (this_one, buf); /* __STRCAT_CHECKED__ */
       }
     }
   } /* while (ftell (in) < off_end) */
 
+  /* Do we have anything pending?  -- XXX, same code as in above in the loop. */
+  if (this_one)
+  {
+    if (flags & CH_DECODE) 
+    {
+      if (!address_header_decode (&this_one))
+	rfc2047_decode (&this_one);
+    }
+    
+    if (!headers[x])
+      headers[x] = this_one;
+    else 
+    {
+      safe_realloc ((void **) &headers[x], mutt_strlen (headers[x]) + 
+		    mutt_strlen (this_one) + sizeof (char));
+      strcat (headers[x], this_one); /* __STRCAT_CHECKED__ */
+      FREE (&this_one);
+    }
+    
+    this_one = NULL;
+  }
+
   /* Now output the headers in order */
   for (x = 0; x < hdr_count; x++)
   {
     if (headers[x])
     {
+#if 0
       if (flags & CH_DECODE)
 	rfc2047_decode (&headers[x]);
+#endif
 
       /* We couldn't do the prefixing when reading because RFC 2047
        * decoding may have concatenated lines.
@@ -546,36 +587,34 @@ _mutt_copy_message (FILE *fpout, FILE *fpin, HEADER *hdr, BODY *body,
     if (flags & M_CM_CHARCONV)
       s.flags |= M_CHARCONV;
     
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-    if (flags & M_CM_VERIFY)
+    if (WithCrypto && flags & M_CM_VERIFY)
       s.flags |= M_VERIFY;
-#endif
 
     mutt_body_handler (body, &s);
   }
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-  else if ((flags & M_CM_DECODE_CRYPT) && (hdr->security & ENCRYPT))
+  else if (WithCrypto
+           && (flags & M_CM_DECODE_CRYPT) && (hdr->security & ENCRYPT))
   {
     BODY *cur;
     FILE *fp;
 
-#ifdef HAVE_PGP
-    if ((flags & M_CM_DECODE_PGP) && (hdr->security & APPLICATION_PGP) &&
+    if ((WithCrypto & APPLICATION_PGP)
+        && (flags & M_CM_DECODE_PGP) && (hdr->security & APPLICATION_PGP) &&
 	hdr->content->type == TYPEMULTIPART)
     {
-      if (pgp_decrypt_mime (fpin, &fp, hdr->content, &cur))
+      if (crypt_pgp_decrypt_mime (fpin, &fp, hdr->content, &cur))
 	return (-1);
       fputs ("Mime-Version: 1.0\n", fpout);
     }
-#endif
-#ifdef HAVE_SMIME
-    if ((flags & M_CM_DECODE_SMIME) && (hdr->security & APPLICATION_SMIME)
+
+    if ((WithCrypto & APPLICATION_SMIME)
+        && (flags & M_CM_DECODE_SMIME) && (hdr->security & APPLICATION_SMIME)
 	     && hdr->content->type == TYPEAPPLICATION)
     {
-      if (smime_decrypt_mime (fpin, &fp, hdr->content, &cur))
+      if (crypt_smime_decrypt_mime (fpin, &fp, hdr->content, &cur))
 	return (-1);
     }
-#endif
+
     mutt_write_mime_header (cur, fpout);
     fputc ('\n', fpout);
 
@@ -589,7 +628,6 @@ _mutt_copy_message (FILE *fpout, FILE *fpin, HEADER *hdr, BODY *body,
     mutt_free_body (&cur);
     fclose (fp);
   }
-#endif
   else
   {
     fseek (fpin, body->offset, 0);
@@ -736,3 +774,152 @@ static int copy_delete_attach (BODY *b, FILE *fpin, FILE *fpout, char *date)
 
   return 0;
 }
+
+/* 
+ * This function is the equivalent of mutt_write_address_list(),
+ * but writes to a buffer instead of writing to a stream.
+ * mutt_write_address_list could be re-used if we wouldn't store
+ * all the decoded headers in a huge array, first. 
+ *
+ * XXX - fix that. 
+ */
+
+static void format_address_header (char **h, ADDRESS *a)
+{
+  char buf[HUGE_STRING];
+  char cbuf[STRING];
+  char c2buf[STRING];
+  
+  int l, linelen, buflen, count;
+  linelen = mutt_strlen (*h);
+  buflen  = linelen + 3;
+  
+  
+  safe_realloc ((void **) h, buflen);
+  for (count = 0; a; a = a->next, count++)
+  {
+    ADDRESS *tmp = a->next;
+    a->next = NULL;
+    *buf = *cbuf = *c2buf = '\0';
+    rfc822_write_address (buf, sizeof (buf), a, 0);
+    a->next = tmp;
+    
+    l = mutt_strlen (buf);
+    if (count && linelen + l > 74) 
+    {
+      strcpy (cbuf, "\n\t");  	/* __STRCPY_CHECKED__ */
+      linelen = l + 8;
+    }
+    else
+    {
+      if (a->mailbox)
+      {
+	strcpy (cbuf, " ");	/* __STRCPY_CHECKED__ */
+	linelen++;
+      }
+      linelen += l;
+    }
+    if (!a->group && a->next && a->next->mailbox)
+    {
+      linelen++;
+      buflen++;
+      strcpy (c2buf, ",");	/* __STRCPY_CHECKED__ */
+    }
+    
+    buflen += l + mutt_strlen (cbuf) + mutt_strlen (c2buf);
+    safe_realloc ((void **) h, buflen);
+    strcat (*h, cbuf);		/* __STRCAT_CHECKED__ */
+    strcat (*h, buf);		/* __STRCAT_CHECKED__ */
+    strcat (*h, c2buf);		/* __STRCAT_CHECKED__ */
+  }
+  
+  /* Space for this was allocated in the beginning of this function. */
+  strcat (*h, "\n");		/* __STRCAT_CHECKED__ */
+}
+
+static int address_header_decode (char **h)
+{
+  char *s = *h;
+  int l;
+
+  ADDRESS *a = NULL;
+
+  switch (tolower (*s))
+  {
+    case 'r': 
+    {
+      if (ascii_strncasecmp (s, "return-path:", 12) == 0)
+      {
+	l = 12;
+	break;
+      }
+      else if (ascii_strncasecmp (s, "reply-to:", 9) == 0)
+      {
+	l = 9;
+	break;
+      }
+      return 0;
+    }
+    case 'f': 
+    {
+      if (ascii_strncasecmp (s, "from:", 5)) 
+	return 0; 
+      l = 5;
+      break;
+    }
+    case 'c':
+    {
+      if (ascii_strncasecmp (s, "cc:", 3))
+	return 0;
+      l = 3;
+      break;
+      
+    }
+    case 'b':
+    {
+      if (ascii_strncasecmp (s, "bcc:", 4))
+	return 0;
+      l = 4;
+      break;
+    }
+    case 's':
+    {
+      if (ascii_strncasecmp (s, "sender:", 7))
+	return 0;
+      l = 7;
+      break;
+    }
+    case 't':
+    {
+      if (ascii_strncasecmp (s, "to:", 3))
+	return 0;
+      l = 3;
+      break;
+    }
+    case 'm':
+    {
+      if (ascii_strncasecmp (s, "mail-followup-to:", 17))
+	return 0;
+      l = 17;
+      break;
+    }
+    default: return 0;    
+  }
+
+  if ((a = rfc822_parse_adrlist (a, s + l + 1)) == NULL)
+    return 0;
+  
+  mutt_addrlist_to_local (a);
+  rfc2047_decode_adrlist (a);
+  
+  *h = safe_calloc (1, l + 2);
+  
+  strfcpy (*h, s, l + 1);
+  
+  format_address_header (h, a);
+
+  rfc822_free_address (&a);
+  
+  FREE (&s);
+  return 1;
+}
