@@ -30,26 +30,10 @@
 #endif
 
 #ifdef USE_IMAP
-#include "imap.h"
+#include "imap_private.h"
 #endif
 
-
-
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
-
-#ifdef HAVE_SMIME
-#include "smime.h"
-#endif
-
-
-
-
-
-
-
-
+#include "mutt_crypt.h"
 
 
 #include <ctype.h>
@@ -94,6 +78,17 @@ static const char *No_visible = N_("No visible messages.");
 				break; \
 			}
 
+#ifdef USE_IMAP 
+/* the error message returned here could be better. */
+#define CHECK_IMAP_ACL(aclbit) if (Context->magic == M_IMAP) \
+		if (mutt_bit_isset (((IMAP_DATA *)Context->data)->capabilities, ACL) \
+		&& !mutt_bit_isset(((IMAP_DATA *)Context->data)->rights,aclbit)){ \
+			mutt_flushinp(); \
+			mutt_error ("Operation not permitted by the IMAP ACL for this mailbox"); \
+			break; \
+		}
+#endif
+
 #define CHECK_ATTACH if(option(OPTATTACHMSG)) \
 		     {\
 			mutt_flushinp (); \
@@ -760,6 +755,11 @@ int mutt_index_menu (void)
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	CHECK_ATTACH;
 	mutt_pattern_func (M_DELETE, _("Delete messages matching: "));
 	menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
@@ -917,6 +917,11 @@ int mutt_index_menu (void)
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	if (mutt_pattern_func (M_UNDELETE, _("Undelete messages matching: ")) == 0)
 	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 	break;
@@ -1219,32 +1224,26 @@ int mutt_index_menu (void)
 	  menu->redraw = REDRAW_MOTION;
 	break;
 
+      case OP_DECRYPT_COPY:
+      case OP_DECRYPT_SAVE:
+        if (!WithCrypto)
+          break;   
+        /* fall thru */
       case OP_COPY_MESSAGE:
       case OP_SAVE:
       case OP_DECODE_COPY:
       case OP_DECODE_SAVE:
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-      case OP_DECRYPT_COPY:
-      case OP_DECRYPT_SAVE:
-#endif
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
         if (mutt_save_message (tag ? NULL : CURHDR,
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 			       (op == OP_DECRYPT_SAVE) ||
-#endif
 			       (op == OP_SAVE) || (op == OP_DECODE_SAVE),
 			       (op == OP_DECODE_SAVE) || (op == OP_DECODE_COPY),
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 			       (op == OP_DECRYPT_SAVE) || (op == OP_DECRYPT_COPY) ||
-#endif
 			       0,
 			       &menu->redraw) == 0 &&
-	    (op == OP_SAVE || op == OP_DECODE_SAVE
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-	     || op == OP_DECRYPT_SAVE
-#endif
-	     ))
+	     (op == OP_SAVE || op == OP_DECODE_SAVE || op == OP_DECRYPT_SAVE)
+	    )
 	{
 	  if (tag)
 	    menu->redraw |= REDRAW_INDEX;
@@ -1270,9 +1269,13 @@ int mutt_index_menu (void)
       case OP_MAIN_NEXT_NEW_THEN_UNREAD:
       case OP_MAIN_PREV_NEW_THEN_UNREAD:
 
+      {
+	int first_unread = -1;
+	int first_new    = -1;
+	
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
-      next_unread_again:
+
 	i = menu->current;
 	menu->current = -1;
 	for (j = 0; j != Context->vcount; j++)
@@ -1299,43 +1302,40 @@ int mutt_index_menu (void)
 
 	  if (CURHDRi->collapsed && (Sort & SORT_MASK) == SORT_THREADS)
 	  {
-	    if ((op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD) &&
-		UNREAD (CURHDRi))
-	    {
-	      menu->current = i;
-	      break;
-	    }
-	    if ((op == OP_MAIN_NEXT_NEW || op == OP_MAIN_PREV_NEW || 
-		 op == OP_MAIN_NEXT_NEW_THEN_UNREAD || op == OP_MAIN_PREV_NEW_THEN_UNREAD) &&
-		UNREAD (CURHDRi) == 1)
-	    {
-	      menu->current = i;
-	      break;
-	    }
+	    if (UNREAD (CURHDRi) && first_unread == -1)
+	      first_unread = i;
+	    if (UNREAD (CURHDRi) == 1 && first_new == -1)
+	      first_new = i;
 	  }
 	  else if ((!CURHDRi->deleted && !CURHDRi->read))
 	  {
-	    if (op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD || !CURHDRi->old)
-	    {
-	      menu->current = i;
-	      break;
-	    }
+	    if (first_unread == -1)
+	      first_unread = i;
+	    if ((!CURHDRi->old) && first_new == -1)
+	      first_new = i;
 	  }
+	  
+	  if ((op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD) &&
+	      first_unread != -1)
+	    break;
+	  if ((op == OP_MAIN_NEXT_NEW || op == OP_MAIN_PREV_NEW ||
+	       op == OP_MAIN_NEXT_NEW_THEN_UNREAD || op == OP_MAIN_PREV_NEW_THEN_UNREAD)
+	      && first_new != -1)
+	    break;
 	}
 #undef CURHDRi
+	if ((op == OP_MAIN_NEXT_NEW || op == OP_MAIN_PREV_NEW ||
+	     op == OP_MAIN_NEXT_NEW_THEN_UNREAD || op == OP_MAIN_PREV_NEW_THEN_UNREAD) 
+	    && first_new != -1)
+	  menu->current = first_new;
+	else if ((op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD ||
+		  op == OP_MAIN_NEXT_NEW_THEN_UNREAD || op == OP_MAIN_PREV_NEW_THEN_UNREAD)
+		 && first_unread != -1)
+	  menu->current = first_unread;
+
 	if (menu->current == -1)
 	{
 	  menu->current = menu->oldcurrent;
-	  if (op == OP_MAIN_NEXT_NEW_THEN_UNREAD) 
-	  {
-	    op = OP_MAIN_NEXT_UNREAD;
-	    goto next_unread_again;
-	  }
-	  if (op == OP_MAIN_PREV_NEW_THEN_UNREAD)
-	  {
-	    op = OP_MAIN_PREV_UNREAD;
-	    goto next_unread_again;
-	  }
 	  mutt_error ("%s%s.", (op == OP_MAIN_NEXT_NEW || op == OP_MAIN_PREV_NEW) ? _("No new messages") : _("No unread messages"),
 		      Context->pattern ? _(" in this limited view") : "");
 	}
@@ -1347,7 +1347,7 @@ int mutt_index_menu (void)
 	else
 	  menu->redraw = REDRAW_MOTION;
 	break;
-
+      }
       case OP_FLAG_MESSAGE:
 
 	CHECK_MSGCOUNT;
@@ -1363,6 +1363,10 @@ int mutt_index_menu (void)
 	}
 #endif
 
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_WRITE);
+#endif
+
         if (tag)
         {
 	  for (j = 0; j < Context->vcount; j++)
@@ -1398,6 +1402,11 @@ int mutt_index_menu (void)
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_SEEN);
+#endif
+
 	if (tag)
 	{
 	  for (j = 0; j < Context->vcount; j++)
@@ -1510,6 +1519,11 @@ int mutt_index_menu (void)
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
+	
+/* #ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_WRITE);
+#endif */
+
 	if (mutt_change_flag (tag ? NULL : CURHDR, (op == OP_MAIN_SET_FLAG)) == 0)
 	{
 	  menu->redraw = REDRAW_STATUS;
@@ -1648,6 +1662,11 @@ int mutt_index_menu (void)
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
+	
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	if (tag)
 	{
 	  mutt_tag_set_flag (M_DELETE, 1);
@@ -1688,6 +1707,10 @@ int mutt_index_menu (void)
         CHECK_VISIBLE;
 	CHECK_READONLY;
 
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	rc = mutt_thread_set_flag (CURHDR, M_DELETE, 1,
 				   op == OP_DELETE_THREAD ? 0 : 1);
 
@@ -1737,6 +1760,10 @@ int mutt_index_menu (void)
 	}
 #endif
 
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_INSERT);
+#endif
+
         mutt_edit_message (Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
 
@@ -1752,15 +1779,9 @@ int mutt_index_menu (void)
 	break;
 
 
-
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       case OP_FORGET_PASSPHRASE:
-
 	crypt_forget_passphrase ();
 	break;
-#endif /* HAVE_PGP */
-
-
 
       case OP_GROUP_REPLY:
 
@@ -1787,36 +1808,28 @@ int mutt_index_menu (void)
 	menu->redraw = REDRAW_FULL;
 	break;
 
-
-
-
-
-
-
-#ifdef HAVE_PGP
       case OP_MAIL_KEY:
-	
+        if (!(WithCrypto & APPLICATION_PGP))
+          break;
 	CHECK_ATTACH;
 	ci_send_message (SENDKEY, NULL, NULL, NULL, NULL);
 	menu->redraw = REDRAW_FULL;
 	break;
-#endif /* HAVE_PGP */
 
       
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       case OP_EXTRACT_KEYS:
-      
+        if (!WithCrypto)
+          break;
         CHECK_MSGCOUNT;
         CHECK_VISIBLE;
         crypt_extract_keys_from_messages(tag ? NULL : CURHDR);
         menu->redraw = REDRAW_FULL;
         break;
 
-#endif /* HAVE_PGP || HAVE_SMIME */
 
-#ifdef HAVE_PGP
       case OP_CHECK_TRADITIONAL:
-      
+        if (!(WithCrypto & APPLICATION_PGP))
+          break;
         CHECK_MSGCOUNT; 
         CHECK_VISIBLE;
         mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
@@ -1827,14 +1840,6 @@ int mutt_index_menu (void)
 	}
         break;
       
-#endif /* HAVE_PGP */
-
-
-
-
-
-
-
       case OP_PIPE:
 
 	CHECK_MSGCOUNT;
@@ -1857,6 +1862,10 @@ int mutt_index_menu (void)
         CHECK_VISIBLE;
 	CHECK_READONLY;
 
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_SEEN);
+#endif
+
 	rc = mutt_thread_set_flag (CURHDR, M_READ, 1,
 				   op == OP_MAIN_READ_THREAD ? 0 : 1);
 
@@ -1939,6 +1948,11 @@ int mutt_index_menu (void)
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+	
 	if (tag)
 	{
 	  mutt_tag_set_flag (M_DELETE, 0);
@@ -1965,6 +1979,10 @@ int mutt_index_menu (void)
         CHECK_VISIBLE;
 	CHECK_READONLY;
 
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	rc = mutt_thread_set_flag (CURHDR, M_DELETE, 0,
 				   op == OP_UNDELETE_THREAD ? 0 : 1);
 
