@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -44,6 +44,8 @@ int hash_string (const unsigned char *s, int n)
 HASH *hash_create (int nelem)
 {
   HASH *table = safe_malloc (sizeof (HASH));
+  if (nelem == 0)
+    nelem = 2;
   table->nelem = nelem;
   table->table = safe_calloc (nelem, sizeof (struct hash_elem *));
   return table;
@@ -144,9 +146,9 @@ void hash_destroy (HASH **ptr, void (*destroy) (void *))
       elem = elem->next;
       if (destroy)
 	destroy (tmp->data);
-      safe_free ((void **) &tmp);
+      FREE (&tmp);
     }
   }
-  safe_free ((void **) &pptr->table);
-  safe_free ((void **) ptr);
+  FREE (&pptr->table);
+  FREE (ptr);
 }
