@@ -91,6 +91,10 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   while ((msgend) >= idata->ctx->hdrmax)
     mx_alloc_memory (idata->ctx);
 
+  oldmsgcount = ctx->msgcount;
+  idata->reopen &= ~IMAP_NEWMAIL_PENDING;
+  idata->newMailCount = 0;
+
   for (msgno = msgbegin; msgno <= msgend ; msgno++)
   {
     if (ReadInc && (!msgno || ((msgno+1) % ReadInc == 0)))
@@ -120,8 +124,6 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
     memset (&h, 0, sizeof (h));
     h.data = safe_calloc (1, sizeof (IMAP_HEADER_DATA));
 
-    oldmsgcount = ctx->msgcount;
-
     /* this DO loop does two things:
      * 1. handles untagged messages, so we can try again on the same msg
      * 2. fetches the tagged response at the end of the last message.
@@ -145,7 +147,6 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       /* update context with message header */
       ctx->hdrs[msgno] = mutt_new_header ();
 
-      ctx->hdrs[msgno]->index = ctx->msgcount;
       ctx->hdrs[msgno]->index = h.sid - 1;
       if (h.sid != ctx->msgcount + 1)
 	dprint (1, (debugfile, "imap_read_headers: msgcount and sequence ID are inconsistent!"));
@@ -174,9 +175,6 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
     while ((rc != IMAP_CMD_OK) && ((mfhrc == -1) ||
       ((msgno + 1) >= fetchlast)));
 
-    if (ctx->msgcount > oldmsgcount)
-      mx_update_context (ctx, ctx->msgcount - oldmsgcount);
-
     if ((mfhrc < -1) || ((rc != IMAP_CMD_CONTINUE) && (rc != IMAP_CMD_OK)))
     {
       imap_free_header_data ((void**) &h.data);
@@ -185,20 +183,22 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       return -1;
     }
 	
-    /* h.data shouldn't be freed here, it is kept in ctx->headers */
-
     /* in case we get new mail while fetching the headers */
     if (idata->reopen & IMAP_NEWMAIL_PENDING)
     {
       msgend = idata->newMailCount - 1;
       while ((msgend) >= ctx->hdrmax)
 	mx_alloc_memory (ctx);
-      idata->status &= ~IMAP_NEWMAIL_PENDING;
+      idata->reopen &= ~IMAP_NEWMAIL_PENDING;
+      idata->newMailCount = 0;
     }
   }
 
   fclose(fp);
 
+  if (ctx->msgcount > oldmsgcount)
+    mx_update_context (ctx, ctx->msgcount - oldmsgcount);
+
   return msgend;
 }
 
@@ -254,7 +254,7 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
   cache->path = safe_strdup (path);
   if (!(msg->fp = safe_fopen (path, "w+")))
   {
-    safe_free ((void**) &cache->path);
+    FREE (&cache->path);
     return -1;
   }
 
@@ -378,9 +378,9 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
   h->content->length = ftell (msg->fp) - h->content->offset;
 
   /* This needs to be done in case this is a multipart message */
-#ifdef HAVE_PGP
-  h->pgp = pgp_query (h->content);
-#endif /* HAVE_PGP */
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  h->security = crypt_query (h->content);
+#endif
 
   mutt_clear_error();
   rewind (msg->fp);
@@ -676,7 +676,7 @@ void imap_free_header_data (void** data)
   /* this should be safe even if the list wasn't used */
   mutt_free_list (&(((IMAP_HEADER_DATA*) *data)->keywords));
 
-  safe_free (data);
+  FREE (data);
 }
 
 /* imap_set_flags: fill out the message header according to the flags from
