@@ -24,17 +24,8 @@
 #include "keymap.h"
 #include "mbyte.h"
 #include "charset.h"
-
-
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
-
-
-#ifdef HAVE_SMIME
-#include "smime.h"
-#endif
-
+#include "mutt_crypt.h"
+#include "mutt_idna.h"
 
 #if defined(USE_SSL) || defined(USE_NSS)
 #include "mutt_ssl.h"
@@ -498,7 +489,8 @@ static int parse_alias (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
   ALIAS *tmp = Aliases;
   ALIAS *last = NULL;
-
+  char *estr = NULL;
+  
   if (!MoreArgs (s))
   {
     strfcpy (err->data, _("alias: no address"), err->dsize);
@@ -539,6 +531,12 @@ static int parse_alias (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
     last->next = tmp;
   else
     Aliases = tmp;
+  if (mutt_addrlist_to_idna (tmp->addr, &estr))
+  {
+    snprintf (err->data, err->dsize, _("Warning: Bad IDN '%s' in alias '%s'.\n"),
+	      estr, tmp->name);
+    return -1;
+  }
   return 0;
 }
 
@@ -681,7 +679,7 @@ static void mutt_set_default (struct option_t *p)
       {
 	char tmp[HUGE_STRING];
 	*tmp = '\0';
-	rfc822_write_address (tmp, sizeof (tmp), *((ADDRESS **) p->data));
+	rfc822_write_address (tmp, sizeof (tmp), *((ADDRESS **) p->data), 0);
 	p->init = (unsigned long) safe_strdup (tmp);
       }
       break;
@@ -912,7 +910,7 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
 	if (DTYPE (MuttVars[idx].type) == DT_ADDR)
 	{
 	  _tmp[0] = '\0';
-	  rfc822_write_address (_tmp, sizeof (_tmp), *((ADDRESS **) MuttVars[idx].data));
+	  rfc822_write_address (_tmp, sizeof (_tmp), *((ADDRESS **) MuttVars[idx].data), 0);
 	  val = _tmp;
 	}
 	else
@@ -1185,11 +1183,10 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
 	case DT_SORT_BROWSER:
 	  map = SortBrowserMethods;
 	  break;
-#ifdef HAVE_PGP
 	case DT_SORT_KEYS:
-	  map = SortKeyMethods;
+          if ((WithCrypto & APPLICATION_PGP))
+            map = SortKeyMethods;
 	  break;
-#endif
 	case DT_SORT_AUX:
 	  map = SortAuxMethods;
 	  break;
@@ -1593,7 +1590,7 @@ int mutt_var_value_complete (char *buffer, size_t len, int pos)
       }
       else if (DTYPE (MuttVars[idx].type) == DT_ADDR)
       {
-	rfc822_write_address (tmp, sizeof (tmp), *((ADDRESS **) MuttVars[idx].data));
+	rfc822_write_address (tmp, sizeof (tmp), *((ADDRESS **) MuttVars[idx].data), 0);
       }
       else if (DTYPE (MuttVars[idx].type) == DT_QUAD)
 	strfcpy (tmp, vals[quadoption (MuttVars[idx].data)], sizeof (tmp));
@@ -1612,11 +1609,12 @@ int mutt_var_value_complete (char *buffer, size_t len, int pos)
 	  case DT_SORT_BROWSER:
 	    map = SortBrowserMethods;
 	    break;
-#ifdef HAVE_PGP
 	  case DT_SORT_KEYS:
-	    map = SortKeyMethods;
+            if ((WithCrypto & APPLICATION_PGP))
+              map = SortKeyMethods;
+            else
+              map = SortMethods;
 	    break;
-#endif
 	  default:
 	    map = SortMethods;
 	    break;
