@@ -26,14 +26,6 @@
 #include "keymap.h"
 #include "url.h"
 
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
-
-#ifdef HAVE_SMIME
-#include "smime.h"
-#endif
-
 #ifdef USE_IMAP
 #include "imap.h"
 #endif
@@ -50,6 +42,8 @@
 #include "dotlock.h"
 #endif
 
+#include "mutt_crypt.h"
+
 #include <dirent.h>
 #include <fcntl.h>
 #include <sys/file.h>
@@ -121,7 +115,7 @@ retry_lock:
       
       snprintf(msg, sizeof(msg), _("Lock count exceeded, remove lock for %s?"),
 	       path);
-      if(retry && mutt_yesorno(msg, 1) == 1)
+      if(retry && mutt_yesorno(msg, M_YES) == M_YES)
       {
 	flags |= DL_FL_FORCE;
 	retry--;
@@ -403,6 +397,10 @@ int mx_get_magic (const char *path)
     snprintf (tmp, sizeof (tmp), "%s/.mew-cache", path);
     if (access (tmp, F_OK) == 0)
       return (M_MH);
+    
+    snprintf (tmp, sizeof (tmp), "%s/.sylpheed_cache", path);
+    if (access (tmp, F_OK) == 0)
+      return (M_MH);
 
     /* 
      * ok, this isn't an mh folder, but mh mode can be used to read
@@ -1569,12 +1567,11 @@ void mx_update_context (CONTEXT *ctx, int new_messages)
   {
     h = ctx->hdrs[msgno];
 
-
-
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
-    /* NOTE: this _must_ be done before the check for mailcap! */
-    h->security = crypt_query (h->content);
-#endif /* HAVE_PGP || HAVE_SMIME */
+    if (WithCrypto)
+    {
+      /* NOTE: this _must_ be done before the check for mailcap! */
+      h->security = crypt_query (h->content);
+    }
 
     if (!ctx->pattern)
     {
