@@ -30,25 +30,10 @@
 #include "mx.h"
 
 #ifdef USE_IMAP
-#include "imap.h"
+#include "imap_private.h"
 #endif
 
-
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
-
-#ifdef HAVE_SMIME
-#include "smime.h"
-#endif
-
-
-
-
-
-
-
-
+#include "mutt_crypt.h"
 
 #include <sys/stat.h>
 #include <ctype.h>
@@ -90,6 +75,17 @@ static const char *Function_not_permitted_in_attach_message_mode = N_("Function
 			break; \
 		     }
 
+#ifdef USE_IMAP 
+/* the error message returned here could be better. */
+#define CHECK_IMAP_ACL(aclbit) if (Context->magic == M_IMAP) \
+		if (mutt_bit_isset (((IMAP_DATA *)Context->data)->capabilities, ACL) \
+		&& !mutt_bit_isset(((IMAP_DATA *)Context->data)->rights,aclbit)){ \
+			mutt_flushinp(); \
+			mutt_error ("Operation not permitted by the IMAP ACL for this mailbox"); \
+			break; \
+		}
+#endif
+
 struct q_class_t
 {
   int length;
@@ -2190,6 +2186,11 @@ search_next:
       case OP_DELETE:
 	CHECK_MODE(IsHeader (extra));
 	CHECK_READONLY;
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	mutt_set_flag (Context, extra->hdr, M_DELETE, 1);
         if (option (OPTDELETEUNTAG))
 	  mutt_set_flag (Context, extra->hdr, M_TAG, 0);
@@ -2206,6 +2207,10 @@ search_next:
 	CHECK_MODE(IsHeader (extra));
 	CHECK_READONLY;
 
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	r = mutt_thread_set_flag (extra->hdr, M_DELETE, 1,
 				  ch == OP_DELETE_THREAD ? 0 : 1);
 
@@ -2334,6 +2339,10 @@ search_next:
 	}
 #endif
 
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_WRITE);
+#endif
+
 	mutt_set_flag (Context, extra->hdr, M_FLAG, !extra->hdr->flagged);
 	redraw = REDRAW_STATUS | REDRAW_INDEX;
 	if (option (OPTRESOLVE))
@@ -2419,9 +2428,13 @@ search_next:
 	redraw = REDRAW_FULL;
 	break;
 
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       case OP_DECRYPT_SAVE:
-#endif
+        if (!WithCrypto)
+        {
+          ch = -1;
+          break;
+        }
+	/* fall through */
       case OP_SAVE:
 	if (IsAttach (extra))
 	{
@@ -2432,24 +2445,21 @@ search_next:
       case OP_COPY_MESSAGE:
       case OP_DECODE_SAVE:
       case OP_DECODE_COPY:
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       case OP_DECRYPT_COPY:
-#endif
+        if (!WithCrypto && ch == OP_DECRYPT_COPY)
+        {
+          ch = -1;
+          break;
+        }
 	CHECK_MODE(IsHeader (extra));
 	if (mutt_save_message (extra->hdr,
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 			       (ch == OP_DECRYPT_SAVE) ||
-#endif			       
 			       (ch == OP_SAVE) || (ch == OP_DECODE_SAVE),
 			       (ch == OP_DECODE_SAVE) || (ch == OP_DECODE_COPY),
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 			       (ch == OP_DECRYPT_SAVE) || (ch == OP_DECRYPT_COPY) ||
-#endif
 			       0,
 			       &redraw) == 0 && (ch == OP_SAVE || ch == OP_DECODE_SAVE
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 						 || ch == OP_DECRYPT_SAVE
-#endif
 						 ))
 	{
 	  if (option (OPTRESOLVE))
@@ -2482,6 +2492,11 @@ search_next:
       case OP_TOGGLE_NEW:
 	CHECK_MODE(IsHeader (extra));
 	CHECK_READONLY;
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_SEEN);
+#endif
+
 	if (extra->hdr->read || extra->hdr->old)
 	  mutt_set_flag (Context, extra->hdr, M_NEW, 1);
 	else if (!first)
@@ -2499,6 +2514,11 @@ search_next:
       case OP_UNDELETE:
 	CHECK_MODE(IsHeader (extra));
 	CHECK_READONLY;
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	mutt_set_flag (Context, extra->hdr, M_DELETE, 0);
 	redraw = REDRAW_STATUS | REDRAW_INDEX;
 	if (option (OPTRESOLVE))
@@ -2513,6 +2533,10 @@ search_next:
 	CHECK_MODE(IsHeader (extra));
 	CHECK_READONLY;
 
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	r = mutt_thread_set_flag (extra->hdr, M_DELETE, 0,
 				  ch == OP_UNDELETE_THREAD ? 0 : 1);
 
@@ -2555,30 +2579,33 @@ search_next:
 	break;
 
 
-
-#ifdef HAVE_PGP
       case OP_MAIL_KEY:
+        if (!(WithCrypto & APPLICATION_PGP))
+        {
+          ch = -1;
+          break;
+        }
 	CHECK_MODE(IsHeader(extra));
         CHECK_ATTACH;
 	ci_send_message (SENDKEY, NULL, NULL, extra->ctx, extra->hdr);
 	redraw = REDRAW_FULL;
 	break;
-#endif /* HAVE_PGP || HAVE_SMIME */
 
 
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       case OP_FORGET_PASSPHRASE:
 	crypt_forget_passphrase ();
 	break;
 
       case OP_EXTRACT_KEYS:
+        if (!WithCrypto)
+        {
+          ch = -1;
+          break;
+        }
         CHECK_MODE(IsHeader(extra));
 	crypt_extract_keys_from_messages(extra->hdr);
         redraw = REDRAW_FULL;
         break;
-#endif /* HAVE_PGP || HAVE_SMIME */
-
-
 
       default:
 	ch = -1;
