@@ -22,17 +22,7 @@
 #include "mime.h"
 #include "rfc2047.h"
 #include "rfc2231.h"
-
-
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif /* HAVE_PGP */
-
-#ifdef HAVE_SMIME
-#include "smime.h"
-#endif /* HAVE_SMIME */
-
-
+#include "mutt_crypt.h"
 
 #include <string.h>
 #include <ctype.h>
@@ -480,7 +470,7 @@ BODY *mutt_read_mime_header (FILE *fp, int digest)
       else if (!ascii_strcasecmp ("encoding-info", line + 6))
         p->encoding = mutt_check_encoding (c);
       else if (!ascii_strcasecmp ("content-lines", line + 6))
-        mutt_set_parameter ("content-lines", safe_strdup (c), &(p->parameter));
+        mutt_set_parameter ("content-lines", c, &(p->parameter));
       else if (!ascii_strcasecmp ("data-description", line + 6))
       {
 	mutt_str_replace (&p->description, c);
@@ -940,9 +930,8 @@ void mutt_parse_mime_message (CONTEXT *ctx, HEADER *cur)
   {
     mutt_parse_part (msg->fp, cur->content);
 
-#if defined(HAVE_PGP) ||  defined(HAVE_SMIME)
-    cur->security = crypt_query (cur->content);
-#endif
+    if (WithCrypto)
+      cur->security = crypt_query (cur->content);
 
     mx_close_message (&msg);
   }
@@ -1265,6 +1254,8 @@ int mutt_parse_rfc822_line (ENVELOPE *e, HEADER *hdr, char *line, char *p, short
  * 		$weed option, honor the header weed list for user headers.
  * 	        Used for recall-message.
  * 
+ * Returns:     newly allocated envelope structure.  You should free it by
+ *              mutt_free_envelope() when envelope stay unneeded.
  */
 ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
 				   short weed)
