@@ -51,9 +51,9 @@
 # include <sys/resource.h>
 #endif
 
-#ifdef HAVE_PGP
+#ifdef CRYPT_BACKEND_CLASSIC_PGP
 
-#include "crypt.h"
+#include "mutt_crypt.h"
 
 
 
@@ -73,8 +73,12 @@ void pgp_forget_passphrase (void)
   mutt_message _("PGP passphrase forgotten.");
 }
 
+int pgp_use_gpg_agent (void)
+{
+  return option (OPTUSEGPGAGENT) && getenv ("GPG_TTY") && getenv ("GPG_AGENT_INFO");
+}
 
-char *pgp_keyid(pgp_key_t *k)
+char *pgp_keyid(pgp_key_t k)
 {
   if((k->flags & KEYFLAG_SUBKEY) && k->parent)
     k = k->parent;
@@ -82,7 +86,7 @@ char *pgp_keyid(pgp_key_t *k)
   return _pgp_keyid(k);
 }
 
-char *_pgp_keyid(pgp_key_t *k)
+char *_pgp_keyid(pgp_key_t k)
 {
   if(option(OPTPGPLONGIDS))
     return k->keyid;
@@ -308,25 +312,27 @@ void pgp_application_pgp_handler (BODY *m, STATE *s)
 	  if (needpass)
 	  {
 	    if (!pgp_valid_passphrase ()) pgp_void_passphrase();
+            if (pgp_use_gpg_agent())
+              *PgpPass = 0;
 	    fprintf (pgpin, "%s\n", PgpPass);
 	  }
 	  
 	  safe_fclose (&pgpin);
 
-	  rv = mutt_wait_filter (thepid);
-
 	  if (s->flags & M_DISPLAY)
 	  {
 	    crypt_current_time (s, "PGP");
 	    rc = pgp_copy_checksig (pgperr, s->fpout);
-	    if (rc == 0) have_any_sigs = 1;
-	    if (rc || rv) maybe_goodsig = 0;
 	  }
 	  
 	  safe_fclose (&pgperr);
+	  rv = mutt_wait_filter (thepid);
 	  
 	  if (s->flags & M_DISPLAY)
 	  {
+	    if (rc == 0) have_any_sigs = 1;
+	    if (rc || rv) maybe_goodsig = 0;
+
 	    state_putc ('\n', s);
 	    state_attach_puts (_("[-- End of PGP output --]\n\n"), s);
 	  }
@@ -361,13 +367,10 @@ void pgp_application_pgp_handler (BODY *m, STATE *s)
 	FGETCONV *fc;
 	int c;
 	rewind (pgpout);
+	state_set_prefix (s);
 	fc = fgetconv_open (pgpout, "utf-8", Charset, 0);
 	while ((c = fgetconv (fc)) != EOF)
-	{
-	  state_putc (c, s);
-	  if (c == '\n' && s->prefix)
-	    state_puts (s->prefix, s);
-	}
+	  state_prefix_putc (c, s);
 	fgetconv_close (&fc);
       }
 
@@ -419,6 +422,7 @@ static int pgp_check_traditional_one_body (FILE *fp, BODY *b, int tagged_only)
   
   short sgn = 0;
   short enc = 0;
+  short key = 0;
   
   if (b->type != TYPETEXT)
     return 0;
@@ -447,18 +451,25 @@ static int pgp_check_traditional_one_body (FILE *fp, BODY *b, int tagged_only)
 	enc = 1;
       else if (mutt_strcmp ("SIGNED MESSAGE-----\n", buf + 15) == 0)
 	sgn = 1;
+      else if (mutt_strcmp ("PUBLIC KEY BLOCK-----\n", buf + 15) == 0)
+	key = 1;
     }
   }
   safe_fclose (&tfp);
   unlink (tempfile);
 
-  if (!enc && !sgn)
+  if (!enc && !sgn && !key)
     return 0;
 
   /* fix the content type */
   
   mutt_set_parameter ("format", "fixed", &b->parameter);
-  mutt_set_parameter ("x-action", enc ? "pgp-encrypted" : "pgp-signed", &b->parameter);
+  if (enc)
+    mutt_set_parameter ("x-action", "pgp-encrypted", &b->parameter);
+  else if (sgn)
+    mutt_set_parameter ("x-action", "pgp-signed", &b->parameter);
+  else if (key)
+    mutt_set_parameter ("x-action", "pgp-keys", &b->parameter);
   
   return 1;
 }
@@ -484,58 +495,6 @@ int pgp_check_traditional (FILE *fp, BODY *b, int tagged_only)
 }
 
      
-int pgp_is_multipart_encrypted (BODY *b)
-{
-  char *p;
-  
-  if (!b || b->type != TYPEMULTIPART ||
-      !b->subtype || ascii_strcasecmp (b->subtype, "encrypted") ||
-      !(p = mutt_get_parameter ("protocol", b->parameter)) ||
-      ascii_strcasecmp (p, "application/pgp-encrypted"))
-    return 0;
-  
-  return PGPENCRYPT;
-}
-
-int mutt_is_application_pgp (BODY *m)
-{
-  int t = 0;
-  char *p;
-  
-  if (m->type == TYPEAPPLICATION)
-  {
-    if (!ascii_strcasecmp (m->subtype, "pgp") || !ascii_strcasecmp (m->subtype, "x-pgp-message"))
-    {
-      if ((p = mutt_get_parameter ("x-action", m->parameter))
-	  && (!ascii_strcasecmp (p, "sign") || !ascii_strcasecmp (p, "signclear")))
-	t |= PGPSIGN;
-
-      if ((p = mutt_get_parameter ("format", m->parameter)) && 
-	  !ascii_strcasecmp (p, "keys-only"))
-	t |= PGPKEY;
-
-      if(!t) t |= PGPENCRYPT;  /* not necessarily correct, but... */
-    }
-
-    if (!ascii_strcasecmp (m->subtype, "pgp-signed"))
-      t |= PGPSIGN;
-
-    if (!ascii_strcasecmp (m->subtype, "pgp-keys"))
-      t |= PGPKEY;
-  }
-  else if (m->type == TYPETEXT && ascii_strcasecmp ("plain", m->subtype) == 0)
-  {
-    if (((p = mutt_get_parameter ("x-mutt-action", m->parameter))
-	 || (p = mutt_get_parameter ("x-action", m->parameter)) 
-	 || (p = mutt_get_parameter ("action", m->parameter)))
-	 && !ascii_strncasecmp ("pgp-sign", p, 8))
-      t |= PGPSIGN;
-    else if (p && !ascii_strncasecmp ("pgp-encrypt", p, 11))
-      t |= PGPENCRYPT;
-  }
-  return t;
-}
-
 
 
 
@@ -764,8 +723,11 @@ BODY *pgp_decrypt_part (BODY *a, STATE *s, FILE *fpout, BODY *p)
     return (NULL);
   }
 
-  /* send the PGP passphrase to the subprocess */
-  fputs (PgpPass, pgpin);
+  /* send the PGP passphrase to the subprocess.  Never do this if the
+     agent is active, because this might lead to a passphrase send as
+     the message. */
+  if (!pgp_use_gpg_agent())
+    fputs (PgpPass, pgpin);
   fputc ('\n', pgpin);
   fclose(pgpin);
   
@@ -955,7 +917,8 @@ BODY *pgp_sign_message (BODY *a)
     return NULL;
   }
   
-  fputs(PgpPass, pgpin);
+  if (!pgp_use_gpg_agent())
+     fputs(PgpPass, pgpin);
   fputc('\n', pgpin);
   fclose(pgpin);
   
@@ -1055,7 +1018,7 @@ char *pgp_findKeys (ADDRESS *to, ADDRESS *cc, ADDRESS *bcc)
   ADDRESS **last = &tmp;
   ADDRESS *p, *q;
   int i;
-  pgp_key_t *k_info, *key;
+  pgp_key_t k_info = NULL, key = NULL;
 
   const char *fqdn = mutt_fqdn (1);
   
@@ -1214,7 +1177,8 @@ BODY *pgp_encrypt_message (BODY *a, char *keylist, int sign)
 
   if (sign)
   {
-    fputs (PgpPass, pgpin);
+    if (!pgp_use_gpg_agent())
+       fputs (PgpPass, pgpin);
     fputc ('\n', pgpin);
   }
   fclose(pgpin);
@@ -1381,6 +1345,8 @@ BODY *pgp_traditional_encryptsign (BODY *a, int flags, char *keylist)
     return NULL;
   }
 
+  if (pgp_use_gpg_agent())
+    *PgpPass = 0;
   if (flags & SIGN)
     fprintf (pgpin, "%s\n", PgpPass);
   fclose (pgpin);
@@ -1452,7 +1418,4 @@ BODY *pgp_traditional_encryptsign (BODY *a, int flags, char *keylist)
   return b;
 }
 
-
-
-
-#endif /* HAVE_PGP */
+#endif /* CRYPT_BACKEND_CLASSIC_PGP */
