@@ -127,11 +127,9 @@ char *mutt_find_hook (int, const char *);
 char *mutt_gecos_name (char *, size_t, struct passwd *);
 char *mutt_gen_msgid (void);
 char *mutt_get_body_charset (char *, size_t, BODY *);
-char *mutt_get_name (ADDRESS *);
+const char *mutt_get_name (ADDRESS *);
 char *mutt_get_parameter (const char *, PARAMETER *);
-#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 char *mutt_crypt_hook (ADDRESS *);
-#endif /* HAVE_PGP */
 char *mutt_make_date (char *, size_t);
 
 const char *mutt_make_version (void);
@@ -231,7 +229,7 @@ void mutt_update_encoding (BODY *a);
 void mutt_update_tree (ATTACHPTR **, short);
 void mutt_version (void);
 void mutt_view_attachments (HEADER *);
-void mutt_write_address_list (ADDRESS *adr, FILE *fp, int linelen);
+void mutt_write_address_list (ADDRESS *adr, FILE *fp, int linelen, int display);
 void mutt_set_virtual (CONTEXT *);
 
 int mutt_addr_is_user (ADDRESS *);
