@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 2000-2001 Edmund Grimley Evans <edmundo@rano.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,6 @@
 
 #include <ctype.h>
 #include <errno.h>
-#include <iconv.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -47,10 +46,10 @@
 
 extern char RFC822Specials[];
 
-typedef size_t (*encoder_t) (char *, const char *, size_t,
+typedef size_t (*encoder_t) (char *, ICONV_CONST char *, size_t,
 			     const char *);
 
-static size_t convert_string (const char *f, size_t flen,
+static size_t convert_string (ICONV_CONST char *f, size_t flen,
 			      const char *from, const char *to,
 			      char **t, size_t *tlen)
 {
@@ -68,7 +67,7 @@ static size_t convert_string (const char *f, size_t flen,
   if (n == (size_t)(-1) || iconv (cd, 0, 0, &ob, &obl) == (size_t)(-1))
   {
     e = errno;
-    safe_free ((void **) &buf);
+    FREE (&buf);
     iconv_close (cd);
     errno = e;
     return (size_t)(-1);
@@ -118,23 +117,23 @@ char *mutt_choose_charset (const char *fromcode, const char *charsets,
     if (!tocode || n < bestn)
     {
       bestn = n;
-      safe_free ((void **) &tocode);
+      FREE (&tocode);
       tocode = t;
       if (d)
       {
-	safe_free ((void **) &e);
+	FREE (&e);
 	e = s;
       }
       else
-	safe_free ((void **) &s);
+	FREE (&s);
       elen = slen;
       if (!bestn)
 	break;
     }
     else
     {
-      safe_free ((void **) &t);
-      safe_free ((void **) &s);
+      FREE (&t);
+      FREE (&s);
     }
   }
   if (tocode)
@@ -150,7 +149,7 @@ char *mutt_choose_charset (const char *fromcode, const char *charsets,
   return tocode;
 }
 
-static size_t b_encoder (char *s, const char *d, size_t dlen,
+static size_t b_encoder (char *s, ICONV_CONST char *d, size_t dlen,
 			 const char *tocode)
 {
   char *s0 = s;
@@ -191,7 +190,7 @@ static size_t b_encoder (char *s, const char *d, size_t dlen,
   return s - s0;
 }
 
-static size_t q_encoder (char *s, const char *d, size_t dlen,
+static size_t q_encoder (char *s, ICONV_CONST char *d, size_t dlen,
 			 const char *tocode)
 {
   char hex[] = "0123456789ABCDEF";
@@ -227,13 +226,13 @@ static size_t q_encoder (char *s, const char *d, size_t dlen,
  * tocode, unless fromcode is 0, in which case the data is assumed to
  * be already in tocode, which should be 8-bit and stateless.
  */
-static size_t try_block (const char *d, size_t dlen,
+static size_t try_block (ICONV_CONST char *d, size_t dlen,
 			 const char *fromcode, const char *tocode,
 			 encoder_t *encoder, size_t *wlen)
 {
   char buf1[ENCWORD_LEN_MAX - ENCWORD_LEN_MIN + 1];
   iconv_t cd;
-  const char *ib;
+  ICONV_CONST char *ib;
   char *ob, *p;
   size_t ibl, obl;
   int count, len, len_b, len_q;
@@ -305,7 +304,7 @@ static size_t encode_block (char *s, char *d, size_t dlen,
 {
   char buf1[ENCWORD_LEN_MAX - ENCWORD_LEN_MIN + 1];
   iconv_t cd;
-  const char *ib;
+  ICONV_CONST char *ib;
   char *ob;
   size_t ibl, obl, n1, n2;
 
@@ -363,7 +362,7 @@ static size_t choose_block (char *d, size_t dlen, int col,
  * The input data is assumed to be a single line starting at column col;
  * if col is non-zero, the preceding character was a space.
  */
-static int rfc2047_encode (const char *d, size_t dlen, int col,
+static int rfc2047_encode (ICONV_CONST char *d, size_t dlen, int col,
 			   const char *fromcode, const char *charsets,
 			   char **e, size_t *elen, char *specials)
 {
@@ -534,8 +533,8 @@ static int rfc2047_encode (const char *d, size_t dlen, int col,
   bufpos += wlen;
   memcpy (buf + bufpos, t1, u + ulen - t1);
 
-  safe_free ((void **) &tocode1);
-  safe_free ((void **) &u);
+  FREE (&tocode1);
+  FREE (&u);
 
   buf[buflen] = '\0';
   
@@ -561,7 +560,7 @@ void _rfc2047_encode_string (char **pd, int encode_specials, int col)
 		  Charset, charsets, &e, &elen,
 		  encode_specials ? RFC822Specials : NULL);
 
-  safe_free ((void **) pd);
+  FREE (pd);
   *pd = e;
 }
 
@@ -584,10 +583,10 @@ void rfc2047_encode_adrlist (ADDRESS *addr, const char *tag)
 
 static int rfc2047_decode_word (char *d, const char *s, size_t len)
 {
-  const char *pp = s, *pp1;
+  const char *pp, *pp1;
   char *pd, *d0;
   const char *t, *t1;
-  int enc = 0, count = 0, c1, c2, c3, c4;
+  int enc = 0, count = 0;
   char *charset = NULL;
 
   pd = d0 = safe_malloc (strlen (s));
@@ -613,65 +612,51 @@ static int rfc2047_decode_word (char *d, const char *s, size_t len)
 	  enc = ENCBASE64;
 	else
 	{
-	  safe_free ((void **) &charset);
-	  safe_free ((void **) &d0);
+	  FREE (&charset);
+	  FREE (&d0);
 	  return (-1);
 	}
 	break;
       case 4:
 	if (enc == ENCQUOTEDPRINTABLE)
 	{
-	  while (pp < pp1 && len > 0)
+	  for (; pp < pp1; pp++)
 	  {
 	    if (*pp == '_')
-	    {
 	      *pd++ = ' ';
-	      len--;
-	    }
-	    else if (*pp == '=')
+	    else if (*pp == '=' &&
+		     (!(pp[1] & ~127) && hexval(pp[1]) != -1) &&
+		     (!(pp[2] & ~127) && hexval(pp[2]) != -1))
 	    {
-	      if (pp[1] == 0 || pp[2] == 0)
-		break;	/* something wrong */
 	      *pd++ = (hexval(pp[1]) << 4) | hexval(pp[2]);
-	      len--;
 	      pp += 2;
 	    }
 	    else
-	    {
 	      *pd++ = *pp;
-	      len--;
-	    }
-	    pp++;
 	  }
 	  *pd = 0;
 	}
 	else if (enc == ENCBASE64)
 	{
-	  while (pp < pp1 && len > 0)
+	  int c, b = 0, k = 0;
+
+	  for (; pp < pp1; pp++)
 	  {
-	    if (pp[0] == '=' || pp[1] == 0 || pp[1] == '=')
-	      break;  /* something wrong */
-	    c1 = base64val(pp[0]);
-	    c2 = base64val(pp[1]);
-	    *pd++ = (c1 << 2) | ((c2 >> 4) & 0x3);
-	    if (--len == 0) break;
-	    
-	    if (pp[2] == 0 || pp[2] == '=') break;
-
-	    c3 = base64val(pp[2]);
-	    *pd++ = ((c2 & 0xf) << 4) | ((c3 >> 2) & 0xf);
-	    if (--len == 0)
+	    if (*pp == '=')
 	      break;
-
-	    if (pp[3] == 0 || pp[3] == '=')
-	      break;
-
-	    c4 = base64val(pp[3]);
-	    *pd++ = ((c3 & 0x3) << 6) | c4;
-	    if (--len == 0)
-	      break;
-
-	    pp += 4;
+	    if ((*pp & ~127) || (c = base64val(*pp)) == -1)
+	      continue;
+	    if (k + 6 >= 8)
+	    {
+	      k -= 2;
+	      *pd++ = b | (c >> k);
+	      b = c << (8 - k);
+	    }
+	    else
+	    {
+	      b |= c << (k + 2);
+	      k += 6;
+	    }
 	  }
 	  *pd = 0;
 	}
@@ -682,8 +667,8 @@ static int rfc2047_decode_word (char *d, const char *s, size_t len)
   if (charset)
     mutt_convert_string (&d0, charset, Charset, M_ICONV_HOOK_FROM);
   strfcpy (d, d0, len);
-  safe_free ((void **) &charset);
-  safe_free ((void **) &d0);
+  FREE (&charset);
+  FREE (&d0);
   return (0);
 }
 
@@ -772,7 +757,7 @@ void rfc2047_decode (char **pd)
   }
   *d = 0;
 
-  safe_free ((void **) pd);
+  FREE (pd);
   *pd = d0;
   mutt_str_adjust (pd);
 }
