@@ -28,7 +28,8 @@
 #include "copy.h"
 #include "pager.h"
 #include "charset.h"
-
+#include "mutt_crypt.h"
+#include "mutt_idna.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -114,16 +115,6 @@ sysexits_h[] =
 };
 
     
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif /* HAVE_PGP */
-
-
-#ifdef HAVE_SMIME
-#include "smime.h"
-#endif /* HAVE_SMIME */
-
-
 
 #define DISPOSITION(X) X==DISPATTACH?"attachment":"inline"
 
@@ -452,11 +443,9 @@ int mutt_write_mime_header (BODY *a, FILE *f)
   return (ferror (f) ? -1 : 0);
 }
 
-#ifdef HAVE_PGP
-# define write_as_text_part(a) (mutt_is_text_part(a) || mutt_is_application_pgp(a))
-#else
-# define write_as_text_part(a) (mutt_is_text_part(a))
-#endif
+# define write_as_text_part(a)  (mutt_is_text_part(a) \
+                                 || ((WithCrypto & APPLICATION_PGP)\
+                                      && mutt_is_application_pgp(a)))
 
 int mutt_write_mime_body (BODY *a, FILE *f)
 {
@@ -490,18 +479,14 @@ int mutt_write_mime_body (BODY *a, FILE *f)
     return (ferror (f) ? -1 : 0);
   }
 
-
-
-#ifdef HAVE_PGP
   /* This is pretty gross, but it's the best solution for now... */
-  if (a->type == TYPEAPPLICATION && mutt_strcmp (a->subtype, "pgp-encrypted") == 0)
+  if ((WithCrypto & APPLICATION_PGP)
+      && a->type == TYPEAPPLICATION
+      && mutt_strcmp (a->subtype, "pgp-encrypted") == 0)
   {
     fputs ("Version: 1\n", f);
     return 0;
   }
-#endif /* HAVE_PGP */
-
-
 
   if ((fpin = fopen (a->filename, "r")) == NULL)
   {
@@ -1311,17 +1296,16 @@ BODY *mutt_make_message_attach (CONTEXT *ctx, HEADER *hdr, int attach_msg)
   BODY *body;
   FILE *fp;
   int cmflags, chflags;
-#if defined(HAVE_PGP) ||  defined(HAVE_SMIME)
-  int pgp = hdr->security;
-#endif
+  int pgp = WithCrypto? hdr->security : 0;
 
-#if defined(HAVE_PGP) ||  defined(HAVE_SMIME)
-  if ((option(OPTMIMEFORWDECODE) || option(OPTFORWDECRYPT)) &&
-      (hdr->security & ENCRYPT)) {
-    if (!crypt_valid_passphrase(hdr->security))
-      return (NULL);
+  if (WithCrypto)
+  {
+    if ((option(OPTMIMEFORWDECODE) || option(OPTFORWDECRYPT)) &&
+        (hdr->security & ENCRYPT)) {
+      if (!crypt_valid_passphrase(hdr->security))
+        return (NULL);
+    }
   }
-#endif /* defined(HAVE_PGP) ||  defined(HAVE_SMIME) */
 
   mutt_mktemp (buffer);
   if ((fp = safe_fopen (buffer, "w+")) == NULL)
@@ -1345,45 +1329,36 @@ BODY *mutt_make_message_attach (CONTEXT *ctx, HEADER *hdr, int attach_msg)
   {
     chflags |= CH_MIME | CH_TXTPLAIN;
     cmflags = M_CM_DECODE | M_CM_CHARCONV;
-#ifdef HAVE_PGP
-    pgp &= ~PGPENCRYPT;
-#endif
-#ifdef HAVE_SMIME
-    pgp &= ~SMIMEENCRYPT;
-#endif
+    if ((WithCrypto & APPLICATION_PGP))
+      pgp &= ~PGPENCRYPT;
+    if ((WithCrypto & APPLICATION_SMIME))
+      pgp &= ~SMIMEENCRYPT;
   }
-#if defined(HAVE_PGP) ||  defined(HAVE_SMIME)
-  else
-    if (option (OPTFORWDECRYPT)
-       && (hdr->security & ENCRYPT))
+  else if (WithCrypto
+           && option (OPTFORWDECRYPT) && (hdr->security & ENCRYPT))
   {
-#ifdef HAVE_PGP
-    if (mutt_is_multipart_encrypted (hdr->content))
+    if ((WithCrypto & APPLICATION_PGP)
+        && mutt_is_multipart_encrypted (hdr->content))
     {
       chflags |= CH_MIME | CH_NONEWLINE;
       cmflags = M_CM_DECODE_PGP;
       pgp &= ~PGPENCRYPT;
     }
-    else if (mutt_is_application_pgp (hdr->content) & PGPENCRYPT)
+    else if ((WithCrypto & APPLICATION_PGP)
+             && (mutt_is_application_pgp (hdr->content) & PGPENCRYPT))
     {
       chflags |= CH_MIME | CH_TXTPLAIN;
       cmflags = M_CM_DECODE | M_CM_CHARCONV;
       pgp &= ~PGPENCRYPT;
     }
-#endif
-#if defined(HAVE_PGP) &&  defined(HAVE_SMIME)
-    else
-#endif
-#ifdef HAVE_SMIME
-    if (mutt_is_application_smime (hdr->content) & SMIMEENCRYPT)
+    else if ((WithCrypto & APPLICATION_SMIME)
+              && mutt_is_application_smime (hdr->content) & SMIMEENCRYPT)
     {
       chflags |= CH_MIME | CH_TXTPLAIN;
       cmflags = M_CM_DECODE | M_CM_CHARCONV;
       pgp &= ~SMIMEENCRYPT;
     }
-#endif
   }
-#endif
 
   mutt_copy_message (fp, ctx, hdr, cmflags, chflags);
   
@@ -1394,9 +1369,8 @@ BODY *mutt_make_message_attach (CONTEXT *ctx, HEADER *hdr, int attach_msg)
   body->hdr->offset = 0;
   /* we don't need the user headers here */
   body->hdr->env = mutt_read_rfc822_header(fp, body->hdr, 0, 0);
-#if defined(HAVE_PGP) ||  defined(HAVE_SMIME)
-  body->hdr->security = pgp;
-#endif /* HAVE_PGP */
+  if (WithCrypto)
+    body->hdr->security = pgp;
   mutt_update_encoding (body);
   body->parts = body->hdr->content;
 
@@ -1524,7 +1498,7 @@ char *mutt_make_date (char *s, size_t len)
 
 /* wrapper around mutt_write_address() so we can handle very large
    recipient lists without needing a huge temporary buffer in memory */
-void mutt_write_address_list (ADDRESS *adr, FILE *fp, int linelen)
+void mutt_write_address_list (ADDRESS *adr, FILE *fp, int linelen, int display)
 {
   ADDRESS *tmp;
   char buf[LONG_STRING];
@@ -1536,15 +1510,12 @@ void mutt_write_address_list (ADDRESS *adr, FILE *fp, int linelen)
     tmp = adr->next;
     adr->next = NULL;
     buf[0] = 0;
-    rfc822_write_address (buf, sizeof (buf), adr);
+    rfc822_write_address (buf, sizeof (buf), adr, display);
     len = mutt_strlen (buf);
     if (count && linelen + len > 74)
     {
-      if (count)
-      {
-	fputs ("\n\t", fp);
-	linelen = len + 8; /* tab is usually about 8 spaces... */
-      }
+      fputs ("\n\t", fp);
+      linelen = len + 8; /* tab is usually about 8 spaces... */
     }
     else
     {
@@ -1600,6 +1571,8 @@ static void write_references (LIST *r, FILE *f)
 /* Note: all RFC2047 encoding should be done outside of this routine, except
  * for the "real name."  This will allow this routine to be used more than
  * once, if necessary.
+ * 
+ * Likewise, all IDN processing should happen outside of this routine.
  *
  * mode == 1  => "lite" mode (used for edit_hdrs)
  * mode == 0  => normal mode.  write full header + MIME headers
@@ -1608,7 +1581,7 @@ static void write_references (LIST *r, FILE *f)
  * privacy != 0 => will omit any headers which may identify the user.
  *               Output generated is suitable for being sent through
  * 		 anonymous remailer chains.
- * 
+ *
  */
 
 int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach, 
@@ -1628,14 +1601,14 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
   if (env->from && !privacy)
   {
     buffer[0] = 0;
-    rfc822_write_address (buffer, sizeof (buffer), env->from);
+    rfc822_write_address (buffer, sizeof (buffer), env->from, 0);
     fprintf (fp, "From: %s\n", buffer);
   }
 
   if (env->to)
   {
     fputs ("To: ", fp);
-    mutt_write_address_list (env->to, fp, 4);
+    mutt_write_address_list (env->to, fp, 4, 0);
   }
   else if (mode > 0)
     fputs ("To: \n", fp);
@@ -1643,7 +1616,7 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
   if (env->cc)
   {
     fputs ("Cc: ", fp);
-    mutt_write_address_list (env->cc, fp, 4);
+    mutt_write_address_list (env->cc, fp, 4, 0);
   }
   else if (mode > 0)
     fputs ("Cc: \n", fp);
@@ -1653,7 +1626,7 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
     if(mode != 0 || option(OPTWRITEBCC))
     {
       fputs ("Bcc: ", fp);
-      mutt_write_address_list (env->bcc, fp, 5);
+      mutt_write_address_list (env->bcc, fp, 5, 0);
     }
   }
   else if (mode > 0)
@@ -1671,7 +1644,7 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
   if (env->reply_to)
   {
     fputs ("Reply-To: ", fp);
-    mutt_write_address_list (env->reply_to, fp, 10);
+    mutt_write_address_list (env->reply_to, fp, 10, 0);
   }
   else if (mode > 0)
     fputs ("Reply-To: \n", fp);
@@ -1679,7 +1652,7 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
   if (env->mail_followup_to)
   {
     fputs ("Mail-Followup-To: ", fp);
-    mutt_write_address_list (env->mail_followup_to, fp, 18);
+    mutt_write_address_list (env->mail_followup_to, fp, 18, 0);
   }
 
   if (mode <= 0)
@@ -2255,7 +2228,7 @@ static int _mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to, const char *r
     fprintf (f, "\nResent-%s", mutt_make_date (date, sizeof(date)));
     fprintf (f, "Resent-Message-ID: %s\n", mutt_gen_msgid());
     fputs ("Resent-To: ", f);
-    mutt_write_address_list (to, f, 11);
+    mutt_write_address_list (to, f, 11, 0);
     mutt_copy_header (fp, h, f, ch_flags, NULL);
     fputc ('\n', f);
     mutt_copy_bytes (fp, f, h->content->length);
@@ -2277,7 +2250,8 @@ int mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to)
   const char *fqdn = mutt_fqdn (1);
   char resent_from[STRING];
   int ret;
-
+  char *err;
+  
   resent_from[0] = '\0';
   from = mutt_default_from ();
 
@@ -2285,8 +2259,13 @@ int mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to)
     rfc822_qualify (from, fqdn);
 
   rfc2047_encode_adrlist (from, "Resent-From");
-  
-  rfc822_write_address (resent_from, sizeof (resent_from), from);
+  if (mutt_addrlist_to_idna (from, &err))
+  {
+    mutt_error (_("Bad IDN %s while preparing resent-from."),
+		err);
+    return -1;
+  }
+  rfc822_write_address (resent_from, sizeof (resent_from), from, 0);
 
   ret = _mutt_bounce_message (fp, h, to, resent_from, from);
 
@@ -2414,9 +2393,9 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post,
 
 
 
-#ifdef HAVE_PGP
   /* (postponment) if the mail is to be signed or encrypted, save this info */
-  if (post && (hdr->security & APPLICATION_PGP))
+  if ((WithCrypto & APPLICATION_PGP)
+      && post && (hdr->security & APPLICATION_PGP))
   {
     fputs ("X-Mutt-PGP: ", msg->fp);
     if (hdr->security & ENCRYPT) 
@@ -2429,11 +2408,10 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post,
     }
     fputc ('\n', msg->fp);
   }
-#endif /* HAVE_PGP */
 
-#ifdef HAVE_SMIME
   /* (postponment) if the mail is to be signed or encrypted, save this info */
-  if (post && (hdr->security & APPLICATION_SMIME))
+  if ((WithCrypto & APPLICATION_SMIME)
+      && post && (hdr->security & APPLICATION_SMIME))
   {
     fputs ("X-Mutt-SMIME: ", msg->fp);
     if (hdr->security & ENCRYPT) {
@@ -2448,7 +2426,6 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post,
     }
     fputc ('\n', msg->fp);
   }
-#endif /* HAVE_SMIME */
 
 #ifdef MIXMASTER
   /* (postponement) if the mail is to be sent through a mixmaster 
