@@ -2,6 +2,7 @@
 
 # Copyright (C) 2001,2002 Oliver Ehli <elmy@acm.org>
 # Copyright (C) 2001 Mike Schiraldi <raldi@research.netsol.com>
+# Copyright (C) 2003 Bjoern Jacke <bjoern@j3e.de>
 #
 #     This program is free software; you can redistribute it and/or modify
 #     it under the terms of the GNU General Public License as published by
@@ -25,6 +26,7 @@ umask 077;
 require "timelocal.pl";
 
 sub usage ();
+sub newfile ($;$$);
 sub mutt_Q ($ );
 sub mycopy ($$);
 
@@ -50,7 +52,11 @@ sub do_verify($$$ );
 # Get the directories mutt uses for certificate/key storage.
 
 my $mutt = $ENV{MUTT_CMDLINE} || 'mutt';
+my $opensslbin = "/usr/bin/openssl";
+my @tempfiles = ();
+my @cert_tmp_file = ();
 
+my $tmpdir;
 my $private_keys_path = mutt_Q 'smime_keys';
 my $certificates_path = mutt_Q 'smime_certificates';
 my $root_certs_path   = mutt_Q 'smime_ca_location';
@@ -77,7 +83,7 @@ elsif(@ARGV == 2 and $ARGV[0] eq "label") {
 }
 elsif(@ARGV == 2 and $ARGV[0] eq "add_cert") {
     my $format = -B $ARGV[1] ? 'DER' : 'PEM'; 
-    my $cmd = "openssl x509 -noout -hash -in $ARGV[1] -inform $format";
+    my $cmd = "$opensslbin x509 -noout -hash -in $ARGV[1] -inform $format";
     my $cert_hash = `$cmd`;
     $? and die "'$cmd' returned $?";
     chomp($cert_hash); 
@@ -100,7 +106,7 @@ elsif( @ARGV == 2 and $ARGV[0] eq "add_p12") {
 
     my $pem_file = "$ARGV[1].pem";
     
-    my $cmd = "openssl pkcs12 -in $ARGV[1] -out $pem_file";
+    my $cmd = "$opensslbin pkcs12 -in $ARGV[1] -out $pem_file";
     system $cmd and die "'$cmd' returned $?";
     
     -e $pem_file and -s $pem_file or die("Conversion of $ARGV[1] failed.");
@@ -113,14 +119,14 @@ elsif( @ARGV == 2 and $ARGV[0] eq "add_p12") {
 elsif(@ARGV == 4 and $ARGV[0] eq "add_chain") {
     my $mailbox;
     my $format = -B $ARGV[2] ? 'DER' : 'PEM'; 
-    my $cmd = "openssl x509 -noout -hash -in $ARGV[2] -inform $format";
+    my $cmd = "$opensslbin x509 -noout -hash -in $ARGV[2] -inform $format";
     my $cert_hash = `$cmd`;
 
     $? and die "'$cmd' returned $?";
 
     $format = -B $ARGV[3] ? 'DER' : 'PEM'; 
 
-    $cmd = "openssl x509 -noout -hash -in $ARGV[3] -inform $format";
+    $cmd = "$opensslbin x509 -noout -hash -in $ARGV[3] -inform $format";
     my $issuer_hash = `$cmd`;
     $? and die "'$cmd' returned $?";
     
@@ -299,15 +305,17 @@ sub list_certs () {
     my $issuer_in;
     my $date1_in;
     my $date2_in;
+    my $cert_tmp_list = newfile("cert_tmp.list","temp");
     
     while (1) {
-        open(TMP_FILE, ">cert_tmp.list")
-            or die "Couldn't open cert_tmp.list: $!";
+	
+        open(TMP_FILE, ">$cert_tmp_list")
+            or die "Couldn't open $cert_tmp_list: $!";
         print TMP_FILE $cert;
         close TMP_FILE;        
 
         my $format = -B $certfile ? 'DER' : 'PEM'; 
-        my $cmd = "openssl x509 -subject -issuer -dates -noout -in cert_tmp.list -inform $format";
+        my $cmd = "$opensslbin x509 -subject -issuer -dates -noout -in $cert_tmp_list -inform $format";
         ($subject_in, $issuer_in, $date1_in, $date2_in) = `$cmd`;
         $? and die "'$cmd' returned $?";
 
@@ -354,7 +362,7 @@ sub list_certs () {
       print "$tab - Matching private key installed -\n";
 
     my $format = -B "$certificates_path/$fields[1]" ? 'DER' : 'PEM'; 
-    my $cmd = "openssl x509 -purpose -noout -in cert_tmp.list -inform $format";
+    my $cmd = "$opensslbin x509 -purpose -noout -in $cert_tmp_list -inform $format";
     my $purpose_in = `$cmd`;
     $? and die "'$cmd' returned $?";
 
@@ -370,8 +378,6 @@ sub list_certs () {
     print "\n";
   }
   
-  unlink 'cert_tmp.list';
-
   close(INDEX);
 }
 
@@ -447,12 +453,12 @@ sub add_certificate ($$$$;$) {
     while(-e "$certificates_path/$$hashvalue.$iter") {
         my ($t1, $t2);
         my $format = -B $filename ? 'DER' : 'PEM'; 
-        my $cmd = "openssl x509 -in $filename -inform $format -fingerprint -noout";
+        my $cmd = "$opensslbin x509 -in $filename -inform $format -fingerprint -noout";
         $t1 = `$cmd`;
         $? and die "'$cmd' returned $?";
 
         $format = -B "$certificates_path/$$hashvalue.$iter" ? 'DER' : 'PEM'; 
-        $cmd = "openssl x509 -in $certificates_path/$$hashvalue.$iter -inform $format -fingerprint -noout";
+        $cmd = "$opensslbin x509 -in $certificates_path/$$hashvalue.$iter -inform $format -fingerprint -noout";
         $t2 = `$cmd`;
         $? and die "'$cmd' returned $?";
         
@@ -470,7 +476,7 @@ sub add_certificate ($$$$;$) {
 
         if ($add_to_index) {
             my $format = -B $filename ? 'DER' : 'PEM'; 
-	    my $cmd = "openssl x509 -in $filename -inform $format -email -noout";
+	    my $cmd = "$opensslbin x509 -in $filename -inform $format -email -noout";
 	    @mailbox = `$cmd`;
 	    $? and die "'$cmd' returned $?";
 
@@ -517,8 +523,10 @@ sub parse_pem (@) {
     my @bag_attribs;
     my $numBags = 0;
 
-    open(CERT_FILE, ">cert_tmp.$cert_iter") 
-        or die "Couldn't open cert_tmp.$cert_iter: $!";
+    $cert_tmp_file[$cert_iter] = newfile("cert_tmp.$cert_iter","temp");
+    my $cert_tmp_iter = $cert_tmp_file[$cert_iter];
+    open(CERT_FILE, ">$cert_tmp_iter") 
+        or die "Couldn't open $cert_tmp_iter: $!";
 
     while($_ = shift(@_)) {
         if(/^Bag Attributes/) {
@@ -559,8 +567,10 @@ sub parse_pem (@) {
                 print CERT_FILE;
                 close(CERT_FILE);
                 $cert_iter++;
-                open(CERT_FILE, ">cert_tmp.$cert_iter")
-                    or die "Couldn't open cert_tmp.$cert_iter: $!";
+	        $cert_tmp_file[$cert_iter] = newfile("cert_tmp.$cert_iter","temp");
+	        $cert_tmp_iter = $cert_tmp_file[$cert_iter];
+                open(CERT_FILE, ">$cert_tmp_iter")
+                    or die "Couldn't open $cert_tmp_iter: $!";
                 next;
             }
         }
@@ -613,8 +623,10 @@ sub handle_pem (@) {
     }
     ($iter > $#pem_contents / 4) and die("Couldn't find matching certificate!");
 
-    mycopy "cert_tmp.$key", "tmp_key";
-    mycopy "cert_tmp.$certificate", "tmp_certificate";
+    my $tmp_key = newfile("tmp_key","temp");
+    mycopy $cert_tmp_file[$key], $tmp_key;
+    my $tmp_certificate = newfile("tmp_certificate","temp");
+    mycopy $cert_tmp_file[$certificate], $tmp_certificate;
 
     # root certificate is self signed
     $iter = 0;
@@ -639,19 +651,21 @@ sub handle_pem (@) {
     # what's left are intermediate certificates.
     $iter = 0;
 
-    unlink "tmp_issuer_cert";
-
     # needs to be set, so we can check it later
     $intermediate = $root_cert;
+    my $tmp_issuer_cert = newfile("tmp_issuer_cert","temp");
     while($iter <= $#pem_contents / 4) {
         if ($iter == $key or $iter == $certificate or $iter == $root_cert) {
             $iter++; 
             next;
         }
 
-# FIXME: This should be done in perl
-        my $cmd = "cat cert_tmp.$iter >> tmp_issuer_cert";
-        system $cmd and die "'$cmd' returned $?";
+	open (IC, ">> $tmp_issuer_cert") or die "can't open $tmp_issuer_cert: $?";
+	my $cert_tmp_iter = $cert_tmp_file[$iter];
+	open (CERT, "< $cert_tmp_iter") or die "can't open $cert_tmp_iter: $?";
+	print IC while (<CERT>);
+	close IC;
+	close CERT;
 
 	# although there may be many, just need to know if there was any
 	$intermediate = $iter;
@@ -661,20 +675,21 @@ sub handle_pem (@) {
 
     # no intermediate certificates ? use root-cert instead (if that was found...)
     if($intermediate == $root_cert) {
-        $root_cert == -1 and
+        if ($root_cert == -1) {
 	  die("No root and no intermediate certificates. Can't continue.");
-        mycopy "cert_tmp.$root_cert", 'tmp_issuer_cert';
+	}
+        mycopy $cert_tmp_file[$root_cert], $tmp_issuer_cert;
     }
 
     my $label = query_label;
 
-    my $format = -B 'tmp_certificate' ? 'DER' : 'PEM'; 
-    my $cmd = "openssl x509 -noout -hash -in tmp_certificate -inform $format";
+    my $format = -B $tmp_certificate ? 'DER' : 'PEM'; 
+    my $cmd = "$opensslbin x509 -noout -hash -in $tmp_certificate -inform $format";
     my $cert_hash = `$cmd`;
     $? and die "'$cmd' returned $?";
 
-    $format = -B 'tmp_issuer_cert' ? 'DER' : 'PEM'; 
-    $cmd = "openssl x509 -noout -hash -in tmp_issuer_cert -inform $format";
+    $format = -B $tmp_issuer_cert ? 'DER' : 'PEM'; 
+    $cmd = "$opensslbin x509 -noout -hash -in $tmp_issuer_cert -inform $format";
     my $issuer_hash = `$cmd`;
     $? and die "'$cmd' returned $?";
 
@@ -682,15 +697,12 @@ sub handle_pem (@) {
 
     # Note: $cert_hash will be changed to reflect the correct filename
     #       within add_cert() ONLY, so these _have_ to get called first..
-    add_certificate("tmp_issuer_cert", \$issuer_hash, 0, $label);
-    @mailbox = &add_certificate("tmp_certificate", \$cert_hash, 1, $label, $issuer_hash); 
+    add_certificate($tmp_issuer_cert, \$issuer_hash, 0, $label);
+    @mailbox = &add_certificate("$tmp_certificate", \$cert_hash, 1, $label, $issuer_hash); 
     foreach $mailbox (@mailbox) {
       chomp($mailbox);
-      add_key("tmp_key", $cert_hash, $mailbox, $label);
+      add_key($tmp_key, $cert_hash, $mailbox, $label);
     }
-    
-    unlink <cert_tmp.*>;
-    unlink <tmp_*>;
 }
 
 
@@ -720,8 +732,9 @@ sub modify_entry ($$$;$ ) {
 
     open(INDEX, "<$path/.index") or  
       die "Couldn't open $path/.index: $!";
-    open(NEW_INDEX, ">$path/.index.tmp") or 
-      die "Couldn't create $path/.index.tmp: $!";
+    my $newindex = newfile("$path/.index.tmp");
+    open(NEW_INDEX, ">$newindex") or 
+      die "Couldn't create $newindex: $!";
 
     while(<INDEX>) {
         @fields = split;
@@ -749,8 +762,8 @@ sub modify_entry ($$$;$ ) {
     close(INDEX);
     close(NEW_INDEX);
 
-    rename "$path/.index.tmp", "$path/.index" 
-        or die "Couldn't rename $path/.index.tmp to $path/.index: $!\n";
+    rename $newindex, "$path/.index" 
+        or die "Couldn't rename $newindex to $path/.index: $!\n";
 
     print "\n";
 }
@@ -831,7 +844,7 @@ sub do_verify($$$) {
     $issuer_path = "$certificates_path/$issuerid";
   }
 
-  my $cmd = "openssl verify $root_certs_switch $root_certs_path -purpose smimesign -purpose smimeencrypt -untrusted $issuer_path $cert_path";
+  my $cmd = "$opensslbin verify $root_certs_switch $root_certs_path -purpose smimesign -purpose smimeencrypt -untrusted $issuer_path $cert_path";
   my $output = `$cmd`;
   $? and die "'$cmd' returned $?";
   chop $output;
@@ -842,7 +855,7 @@ sub do_verify($$$) {
   $result eq 'i' and return $result;
 
   my $format = -B $cert_path ? 'DER' : 'PEM'; 
-  $cmd = "openssl x509 -dates -serial -noout -in $cert_path -inform $format";
+  $cmd = "$opensslbin x509 -dates -serial -noout -in $cert_path -inform $format";
   (my $date1_in, my $date2_in, my $serial_in) = `$cmd`;
   $? and die "'$cmd' returned $?";
 
@@ -874,7 +887,7 @@ sub do_verify($$$) {
     
   if ( defined $crl ) {
     my @serial = split (/\=/, $serial_in);
-    my $cmd = "openssl crl -text -noout -in $crl | grep -A1 $serial[1]";
+    my $cmd = "$opensslbin crl -text -noout -in $crl | grep -A1 $serial[1]";
     (my $l1, my $l2) = `$cmd`;
     $? and die "'$cmd' returned $?";
     
@@ -900,7 +913,7 @@ sub add_root_cert ($) {
 
   my $format = -B $root_cert ? 'DER' : 'PEM'; 
 
-  my $cmd = "openssl x509 -noout -hash -in $root_cert -inform $format";
+  my $cmd = "$opensslbin x509 -noout -hash -in $root_cert -inform $format";
   my $root_hash = `$cmd`;
   $? and die "'$cmd' returned $?";
 
@@ -912,11 +925,11 @@ sub add_root_cert ($) {
     open(ROOT_CERTS, ">>$root_certs_path") or 
       die ("Couldn't open $root_certs_path for writing");
 
-    $cmd = "openssl x509 -in $root_cert -inform $format -fingerprint -noout";
+    $cmd = "$opensslbin x509 -in $root_cert -inform $format -fingerprint -noout";
     $? and die "'$cmd' returned $?";
     chomp(my $md5fp = `$cmd`);
 
-    $cmd = "openssl x509 -in $root_cert -inform $format -text -noout";
+    $cmd = "$opensslbin x509 -in $root_cert -inform $format -text -noout";
     $? and die "'$cmd' returned $?";
     my @cert_text = `$cmd`;
 
@@ -926,7 +939,7 @@ sub add_root_cert ($) {
     my $line = "=======================================\n";
     print ROOT_CERTS "\n$input$line$md5fp\nPEM-Data:\n";
 
-    $cmd = "openssl x509 -in $root_cert -inform $format";
+    $cmd = "$opensslbin x509 -in $root_cert -inform $format";
     my $cert = `$cmd`;
     $? and die "'$cmd' returned $?";
     print ROOT_CERTS $cert;
@@ -936,3 +949,34 @@ sub add_root_cert ($) {
   
 }
 
+sub newfile ($;$$) {
+	# returns a file name which does not exist for tmp file creation
+	my $filename = shift;
+	my $option = shift;
+	if (! $tmpdir and $option eq "temp") {
+		$tmpdir = mutt_Q 'tmpdir';
+		$tmpdir = newfile("$tmpdir/smime");
+		mkdir $tmpdir, 0700 || die "Can't create $tmpdir: $!\n";
+	}
+	$filename = "$tmpdir/$filename" if ($option eq "temp");
+	my $newfilename = $filename;
+	my $count = 0;
+	while (-e $newfilename) {
+		$newfilename = "$filename.$count";
+		$count++;
+	}
+	unshift(@tempfiles,$newfilename);
+	return $newfilename;
+}
+
+
+END {
+	# remove all our temporary files in the end:
+	for (@tempfiles){
+		if (-f) {
+			unlink;
+		} elsif (-d) { 
+			rmdir;
+		}
+	}
+}
